==> senior_design_system-3.0.4.tar.gz: lib/components/senior_image_cropper/senior_image_cropper_widget.dart <==
import 'dart:async';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:image_crop/image_crop.dart';
import 'package:image_picker/image_picker.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:provider/provider.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

import '../../senior_design_system.dart';

const double BUTTON_MAX_WIDTH = 600;

enum AppBarStyle {
  appbar,
  colorfulHeader,
}

class SeniorImageCropper {
  /// Creates an image cropper component that lets you crop images.
  ///
  /// The [takePhotoLabel], [pickImageLabel], [cropperTitle], [cropImageLabel], [cancelCropLabel],
  /// [onCroppImage], [onPermissionCameraDenied] and [onPermissionGalleryDenied] parameters are required.
  const SeniorImageCropper({
    required this.takePhotoLabel,
    required this.pickImageLabel,
    required this.cropperTitle,
    required this.cropImageLabel,
    required this.cancelCropLabel,
    this.appBarStyle = AppBarStyle.appbar,
    required this.onCroppImage,
    this.maxSizeMb,
    required this.onPermissionCameraDenied,
    required this.onPermissionGalleryDenied,
  });

  /// The Label for the option to take a photo with your smartphone camera.
  final String takePhotoLabel;

  /// The Label for the option to choose a saved image.
  final String pickImageLabel;

  /// Title for the image cropper.
  final String cropperTitle;

  /// The crop image option label.
  final String cropImageLabel;

  /// The label for the option to cancel crop image.
  final String cancelCropLabel;

  /// Image cropper app bar type.
  /// It can be [AppBarStyle.appbar] and [AppBarStyle.colorfulHeader].
  ///
  /// The default value is [AppBarStyle.appbar].
  final AppBarStyle appBarStyle;

  /// Maximum image size.
  final int? maxSizeMb;

  /// Callback function executed when the image is cropped.
  /// Receive the cropped image.
  final Function(File croppedImage) onCroppImage;

  /// Callback function executed when there is no permission to access the camera.
  final Function() onPermissionCameraDenied;

  /// Callback function executed when there is no permission to access the gallery.
  final Function() onPermissionGalleryDenied;

  void _loadPicker({
    required ImageSource source,
    required BuildContext context,
  }) async {
    if (Theme.of(context).platform == TargetPlatform.iOS) {
      if (source == ImageSource.camera && !await Permission.camera.request().isGranted) {
        Navigator.pop(context);
        return onPermissionCameraDenied();
      }

      if (source == ImageSource.gallery && !await Permission.photos.request().isGranted) {
        Navigator.pop(context);
        return onPermissionGalleryDenied();
      }
    }

    final XFile? picked = await ImagePicker().pickImage(
      source: source,
    );

    if (picked != null) {
      Navigator.pop(context);
      Navigator.push(
        context,
        MaterialPageRoute(
          builder: (context) => _CropperView(
            pickedImage: File(picked.path),
            cropperTitle: cropperTitle,
            cropImageLabel: cropImageLabel,
            cancelCropLabel: cancelCropLabel,
            appBarStyle: appBarStyle,
            onCropImage: (croppedImage) async {
              final file = File(croppedImage.path);
              final maxLength = maxSizeMb! * 1024 * 1024;

              if (maxSizeMb != null) {
                while (file.lengthSync() > maxLength) {
                  final compressedImage = await FlutterImageCompress.compressWithList(
                    file.readAsBytesSync(),
                    quality: 50,
                    format: CompressFormat.jpeg,
                  );

                  await file.writeAsBytes(
                    compressedImage,
                    flush: true,
                    mode: FileMode.write,
                  );
                }
              }
              onCroppImage(file);
            },
          ),
        ),
      );
    }
  }

  void pickImage(BuildContext context) {
    showModalBottomSheet<void>(
      context: context,
      builder: (BuildContext context) {
        return Container(
          color: Provider.of<ThemeRepository>(context).theme.colorfulHeaderStructureTheme!.style!.bodyColor,
          child: Padding(
            padding: EdgeInsets.only(
              top: SeniorSpacing.normal,
              left: SeniorSpacing.normal,
              right: SeniorSpacing.normal,
              bottom: SeniorSpacing.normal + MediaQuery.of(context).padding.bottom,
            ),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              mainAxisSize: MainAxisSize.min,
              children: [
                SeniorButton.icon(
                  label: takePhotoLabel,
                  icon: FontAwesomeIcons.camera,
                  type: SeniorButtonType.primary,
                  fullLength: MediaQuery.of(context).size.width < BUTTON_MAX_WIDTH,
                  onPressed: () {
                    _loadPicker(
                      source: ImageSource.camera,
                      context: context,
                    );
                  },
                ),
                const SizedBox(
                  height: SeniorSpacing.medium,
                ),
                SeniorButton.icon(
                  label: pickImageLabel,
                  icon: FontAwesomeIcons.solidFolder,
                  type: SeniorButtonType.secondary,
                  fullLength: MediaQuery.of(context).size.width < BUTTON_MAX_WIDTH,
                  onPressed: () {
                    _loadPicker(
                      source: ImageSource.gallery,
                      context: context,
                    );
                  },
                ),
              ],
            ),
          ),
        );
      },
    );
  }
}

class _CropperView extends StatelessWidget {
  final cropKey = GlobalKey<CropState>();
  final File pickedImage;
  final String cropperTitle;
  final String cropImageLabel;
  final String cancelCropLabel;
  final AppBarStyle appBarStyle;
  final Function(File croppedImage) onCropImage;
  final double maximumScale;

  _CropperView({
    Key? key,
    required this.pickedImage,
    required this.cropperTitle,
    required this.cropImageLabel,
    required this.cancelCropLabel,
    required this.appBarStyle,
    required this.onCropImage,
    // ignore: unused_element
    this.maximumScale = 2.0,
  }) : super(key: key);

  Future<void> _cropImage(BuildContext context) async {
    final scale = cropKey.currentState!.scale;
    final area = cropKey.currentState!.area;

    if (area == null) {
      return;
    }

    final sample = await ImageCrop.sampleImage(
      file: pickedImage,
      preferredSize: (2000 / scale).round(),
    );

    final croppedImage = await ImageCrop.cropImage(
      file: sample,
      area: area,
    );

    onCropImage(croppedImage);
    sample.delete();
    Navigator.pop(context);
  }

  void _cancelCrop(BuildContext context) {
    pickedImage.delete();
    Navigator.pop(context);
  }

  AppBar _buildAppBar(BuildContext context) {
    switch (appBarStyle) {
      case AppBarStyle.colorfulHeader:
        return AppBar(
          title: Text(
            cropperTitle,
            style: SeniorTypography.body(
              color: SeniorColors.pureWhite,
            ),
          ),
          backgroundColor: SeniorColors.pureBlack,
          leading: IconButton(
            icon: const Icon(FontAwesomeIcons.chevronLeft),
            onPressed: () {
              _cancelCrop(context);
            },
            iconSize: SeniorSpacing.normal,
          ),
        );
      case AppBarStyle.appbar:
      default:
        return AppBar(
          title: Text(
            cropperTitle,
            style: SeniorTypography.h4(
              color: SeniorColors.pureWhite,
            ),
          ),
          backgroundColor: SeniorColors.pureBlack,
          leading: IconButton(
            icon: const Icon(FontAwesomeIcons.chevronLeft),
            onPressed: () {
              _cancelCrop(context);
            },
            iconSize: SeniorSpacing.medium,
          ),
        );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: _buildAppBar(context),
      body: Container(
        color: SeniorColors.pureBlack,
        child: Column(
          children: <Widget>[
            Expanded(
              child: Padding(
                padding: const EdgeInsets.symmetric(
                  horizontal: SeniorSpacing.medium,
                  vertical: SeniorSpacing.xbig,
                ),
                child: Crop.file(
                  pickedImage,
                  key: cropKey,
                  alwaysShowGrid: true,
                  maximumScale: maximumScale,
                ),
              ),
            ),
            Container(
              decoration: const BoxDecoration(
                color: SeniorColors.pureWhite,
              ),
              padding: EdgeInsets.only(
                top: SeniorSpacing.normal,
                left: SeniorSpacing.normal,
                right: SeniorSpacing.normal,
                bottom: SeniorSpacing.normal + MediaQuery.of(context).padding.bottom,
              ),
              alignment: AlignmentDirectional.center,
              child: Column(
                mainAxisAlignment: MainAxisAlignment.spaceAround,
                children: <Widget>[
                  SeniorButton(
                    label: cropImageLabel,
                    type: SeniorButtonType.primary,
                    fullLength: MediaQuery.of(context).size.width < BUTTON_MAX_WIDTH,
                    onPressed: () {
                      _cropImage(context);
                    },
                  ),
                  const SizedBox(
                    height: SeniorSpacing.small,
                  ),
                  SeniorButton(
                    label: cancelCropLabel,
                    type: SeniorButtonType.ghost,
                    fullLength: MediaQuery.of(context).size.width < BUTTON_MAX_WIDTH,
                    onPressed: () {
                      _cancelCrop(context);
                    },
                  ),
                ],
              ),
            )
          ],
        ),
      ),
    );
  }
}

==> senior_design_system-3.2.1.tar.gz: lib/components/senior_image_cropper/senior_image_cropper_widget.dart <==
import 'dart:async';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:image_crop/image_crop.dart';
import 'package:image_picker/image_picker.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:provider/provider.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

import '../../senior_design_system.dart';

const double BUTTON_MAX_WIDTH = 600;

enum AppBarStyle {
  appbar,
  colorfulHeader,
}

class SeniorImageCropper {
  /// Creates an image cropper component that lets you crop images.
  ///
  /// The [takePhotoLabel], [pickImageLabel], [cropperTitle], [cropImageLabel], [cancelCropLabel],
  /// [onCroppImage], [onPermissionCameraDenied] and [onPermissionGalleryDenied] parameters are required.
  const SeniorImageCropper({
    required this.takePhotoLabel,
    required this.pickImageLabel,
    required this.cropperTitle,
    required this.cropImageLabel,
    required this.cancelCropLabel,
    this.appBarStyle = AppBarStyle.appbar,
    required this.onCroppImage,
    this.maxSizeMb,
    required this.onPermissionCameraDenied,
    required this.onPermissionGalleryDenied,
  });

  /// The Label for the option to take a photo with your smartphone camera.
  final String takePhotoLabel;

  /// The Label for the option to choose a saved image.
  final String pickImageLabel;

  /// Title for the image cropper.
  final String cropperTitle;

  /// The crop image option label.
  final String cropImageLabel;

  /// The label for the option to cancel crop image.
  final String cancelCropLabel;

  /// Image cropper app bar type.
  /// It can be [AppBarStyle.appbar] and [AppBarStyle.colorfulHeader].
  ///
  /// The default value is [AppBarStyle.appbar].
  final AppBarStyle appBarStyle;

  /// Maximum image size.
  final int? maxSizeMb;

  /// Callback function executed when the image is cropped.
  /// Receive the cropped image.
  final Function(File croppedImage) onCroppImage;

  /// Callback function executed when there is no permission to access the camera.
  final Function() onPermissionCameraDenied;

  /// Callback function executed when there is no permission to access the gallery.
  final Function() onPermissionGalleryDenied;

  void _loadPicker({
    required ImageSource source,
    required BuildContext context,
  }) async {
    if (Theme.of(context).platform == TargetPlatform.iOS) {
      if (source == ImageSource.camera && !await Permission.camera.request().isGranted) {
        Navigator.pop(context);
        return onPermissionCameraDenied();
      }

      if (source == ImageSource.gallery && !await Permission.photos.request().isGranted) {
        Navigator.pop(context);
        return onPermissionGalleryDenied();
      }
    }

    final XFile? picked = await ImagePicker().pickImage(
      source: source,
    );

    if (picked != null) {
      Navigator.pop(context);
      Navigator.push(
        context,
        MaterialPageRoute(
          builder: (context) => _CropperView(
            pickedImage: File(picked.path),
            cropperTitle: cropperTitle,
            cropImageLabel: cropImageLabel,
            cancelCropLabel: cancelCropLabel,
            appBarStyle: appBarStyle,
            onCropImage: (croppedImage) async {
              final file = File(croppedImage.path);
              final maxLength = maxSizeMb! * 1024 * 1024;

              if (maxSizeMb != null) {
                while (file.lengthSync() > maxLength) {
                  final compressedImage = await FlutterImageCompress.compressWithList(
                    file.readAsBytesSync(),
                    quality: 50,
                    format: CompressFormat.jpeg,
                  );

                  await file.writeAsBytes(
                    compressedImage,
                    flush: true,
                    mode: FileMode.write,
                  );
                }
              }
              onCroppImage(file);
            },
          ),
        ),
      );
    }
  }

  void pickImage(BuildContext context) {
    showModalBottomSheet<void>(
      context: context,
      builder: (BuildContext context) {
        return Container(
          color: Provider.of<ThemeRepository>(context).theme.colorfulHeaderStructureTheme!.style!.bodyColor,
          child: Padding(
            padding: EdgeInsets.only(
              top: SeniorSpacing.normal,
              left: SeniorSpacing.normal,
              right: SeniorSpacing.normal,
              bottom: SeniorSpacing.normal + MediaQuery.of(context).padding.bottom,
            ),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              mainAxisSize: MainAxisSize.min,
              children: [
                SeniorButton.icon(
                  label: takePhotoLabel,
                  icon: FontAwesomeIcons.camera,
                  type: SeniorButtonType.primary,
                  fullLength: MediaQuery.of(context).size.width < BUTTON_MAX_WIDTH,
                  onPressed: () {
                    _loadPicker(
                      source: ImageSource.camera,
                      context: context,
                    );
                  },
                ),
                const SizedBox(
                  height: SeniorSpacing.medium,
                ),
                SeniorButton.icon(
                  label: pickImageLabel,
                  icon: FontAwesomeIcons.solidFolder,
                  type: SeniorButtonType.secondary,
                  fullLength: MediaQuery.of(context).size.width < BUTTON_MAX_WIDTH,
                  onPressed: () {
                    _loadPicker(
                      source: ImageSource.gallery,
                      context: context,
                    );
                  },
                ),
              ],
            ),
          ),
        );
      },
    );
  }
}

class _CropperView extends StatelessWidget {
  final cropKey = GlobalKey<CropState>();
  final File pickedImage;
  final String cropperTitle;
  final String cropImageLabel;
  final String cancelCropLabel;
  final AppBarStyle appBarStyle;
  final Function(File croppedImage) onCropImage;
  final double maximumScale;

  _CropperView({
    Key? key,
    required this.pickedImage,
    required this.cropperTitle,
    required this.cropImageLabel,
    required this.cancelCropLabel,
    required this.appBarStyle,
    required this.onCropImage,
    // ignore: unused_element
    this.maximumScale = 2.0,
  }) : super(key: key);

  Future<void> _cropImage(BuildContext context) async {
    final scale = cropKey.currentState!.scale;
    final area = cropKey.currentState!.area;

    if (area == null) {
      return;
    }

    final sample = await ImageCrop.sampleImage(
      file: pickedImage,
      preferredSize: (2000 / scale).round(),
    );

    final croppedImage = await ImageCrop.cropImage(
      file: sample,
      area: area,
    );

    onCropImage(croppedImage);
    sample.delete();
    Navigator.pop(context);
  }

  void _cancelCrop(BuildContext context) {
    pickedImage.delete();
    Navigator.pop(context);
  }

  AppBar _buildAppBar(BuildContext context) {
    switch (appBarStyle) {
      case AppBarStyle.colorfulHeader:
        return AppBar(
          title: Text(
            cropperTitle,
            style: SeniorTypography.body(
              color: SeniorColors.pureWhite,
            ),
          ),
          backgroundColor: SeniorColors.pureBlack,
          leading: IconButton(
            icon: const Icon(FontAwesomeIcons.chevronLeft),
            onPressed: () {
              _cancelCrop(context);
            },
            iconSize: SeniorSpacing.normal,
          ),
        );
      case AppBarStyle.appbar:
      default:
        return AppBar(
          title: Text(
            cropperTitle,
            style: SeniorTypography.h4(
              color: SeniorColors.pureWhite,
            ),
          ),
          backgroundColor: SeniorColors.pureBlack,
          leading: IconButton(
            icon: const Icon(FontAwesomeIcons.chevronLeft),
            onPressed: () {
              _cancelCrop(context);
            },
            iconSize: SeniorSpacing.medium,
          ),
        );
    }
  }

  @override
  Widget build(BuildContext context) {
    final theme = Provider.of<ThemeRepository>(context).theme;

    return Scaffold(
      appBar: _buildAppBar(context),
      body: Container(
        color: SeniorColors.pureBlack,
        child: Column(
          children: <Widget>[
            Expanded(
              child: Padding(
                padding: const EdgeInsets.symmetric(
                  horizontal: SeniorSpacing.medium,
                  vertical: SeniorSpacing.xbig,
                ),
                child: Crop.file(
                  pickedImage,
                  key: cropKey,
                  alwaysShowGrid: true,
                  maximumScale: maximumScale,
                ),
              ),
            ),
            Container(
              decoration: BoxDecoration(
                color: theme.imageCropperTheme?.style?.modalBackground ?? SeniorColors.grayscale0,
              ),
              padding: EdgeInsets.only(
                top: SeniorSpacing.normal,
                left: SeniorSpacing.normal,
                right: SeniorSpacing.normal,
                bottom: SeniorSpacing.normal + MediaQuery.of(context).padding.bottom,
              ),
              alignment: AlignmentDirectional.center,
              child: Column(
                mainAxisAlignment: MainAxisAlignment.spaceAround,
                children: <Widget>[
                  SeniorButton(
                    label: cropImageLabel,
                    type: SeniorButtonType.primary,
                    fullLength: MediaQuery.of(context).size.width < BUTTON_MAX_WIDTH,
                    onPressed: () {
                      _cropImage(context);
                    },
                  ),
                  const SizedBox(
                    height: SeniorSpacing.small,
                  ),
                  SeniorButton(
                    label: cancelCropLabel,
                    type: SeniorButtonType.ghost,
                    fullLength: MediaQuery.of(context).size.width < BUTTON_MAX_WIDTH,
                    onPressed: () {
                      _cancelCrop(context);
                    },
                  ),
                ],
              ),
            )
          ],
        ),
      ),
    );
  }
}
